/*

This program and source is licensed under the 

	GNU GENERAL PUBLIC LICENSE 
	
as in the included LICENSE.TXT or located at
http://www.gnu.org/licenses/gpl.txt

*/

#include "ecu.h"


//
//  Note that the NAME fields are NOT unique.  Could be duplicate entries for same field if e.g.
//  calculation for a particular field changes from ECU to ECU.  Only guaranteed unique field
//  in the table is the first one (ID).
//

ECU_FIELD_DESCRIPTOR EcuFieldTable[] = {

    // ID                                                                 -      *     /

    {   1,  9, 6, Num8,  0, "Coolant Temp",           "CLTMP", "Deg.C",  40,    1.0,   1.0  },
    {   2,  9, 5, Num8,  0, "AF Correction #1",       "AFC1",  "%",      128,   1.0,   1.28 },
    {   3,  9, 4, Num8,  0, "AF Learning #1",         "AFL1",  "%",      128,   1.0,   1.28 },
    {   4,  9, 3, Num8,  0, "AF Correction #2",       "AFC2",  "%",      128,   1.0,   1.28 },
    {   5,  9, 2, Num8,  0, "AF Learning #2",         "AFL2",  "%",      128,   1.0,   1.28 },
    {   6,  9, 1, Num8,  0, "Absolute Manifold Pres.","MANABS","PSIg",     0,   37.0,255.0 },
    {   7,  9, 0, Num16, 0, "Engine Speed",           "RPM",   "/100",     0,   1.0, 400.0 }, // div 4 = RPM, scaled down for ease of charting
    {   8,  9, 7, Num8,  0, "Engine Load",            "LOAD",  "%",        0, 100.0, 255.0 }, // not supported by wrx?

    {   9, 10, 7, Num8,  0, "Vehicle Speed",          "SPEED", "Mph",      0,   1.0,   1.6 },
    {  10, 10, 6, Num8,  0, "Ignition Timing",        "IGNTM", "deg",    128,   1.0,   2.0 },
    {  11, 10, 5, Num8,  0, "Intake Air Temp",        "IATMP", "Deg.C",   40,   1.0,   1.0 },
    {  12, 10, 4, Num16, 0, "Mass Air Flow",          "MAF",   "gram/s",   0,   1.0, 100.0 },
    {  13, 10, 3, Num8,  0, "Throttle open angle",    "TPS",   "%",        0, 100.0, 255.0 },
    {  14, 10, 2, Num16, 0, "Front O2 sensor #1",     "F-O2-1","vg",       0, 0.005,   1.0 },
    {  15, 10, 1, Num16, 0, "Rear O2 sensor",         "R-O2",  "v",        0, 0.005,   1.0 },
    {  16, 10, 0, Num16, 0, "Front O2 sensor #2",     "F-O2-2","v",        0, 0.005,   1.0 },

    {  17, 11, 7, Num8,  0, "Battery ",               "BATT",  "v"  ,      0,   0.8,   1.0 },
    {  18, 11, 6, Num8,  0, "Air flow sensor",        "MAF-V", "v",        0,   0.2,   1.0 },
    {  19, 11, 5, Num8,  0, "Throttle sensor",        "TPS-V", "v",        0,   0.2,   1.0 },
    {  20, 11, 4, Num8,  0, "Diff pressure sensor",   "DIFF-V","v",        0,   0.2,   1.0 },
    {  21, 11, 3, Num8,  0, "Fuel Injector 1 P/W",    "FI1PW", "ms",       0,   0.256, 1.0 },
    {  22, 11, 2, Num8,  0, "Fuel Injector 2 P/W",    "FI2PW", "ms",       0,   0.256, 1.0 },
    {  23, 11, 1, Num8,  0, "Knock Correction",       "KNKC",  "deg",    128,   0.5,   1.0 },
    {  24, 11, 0, Num8,  0, "Atmospheric Pressure",   "ATMOS", "PSIg",     0,  37.0, 255.0 },

    {  25, 12, 7, Num8,  0, "Manifold rel.pressure",  "BOOST", "PSIg",   128,  37.0, 255.0 },
    {  26, 12, 5, Num8,  0, "Fuel Tank Pressure",     "FUELPR","PSIg",   128, 0.0035,  1.0 },
    {  27, 12, 1, Num8,  0, "Fuel Temperature",       "FUELTM","Deg.C",   40,   1.0,   1.0 },

    {  28, 13, 5, Num8,  0, "Fuel Level",             "FUELLV","v",        0,   0.2,   1.0 },
    {  29, 13, 3, Num8,  0, "Pri.Wastegate Duty",     "WGPRI", "%",        0, 100.0, 255.0 },
    {  30, 13, 1, Num8,  0, "CPC Valve  Duty",        "CPCDC", "%",        0,   1.0,   2.55},
    {  31, 13, 0, Num8,  0, "Tumble Pos Rt.",         "TMBLRT","v",        0,   0.2,   1.0 },

    {  32, 14, 7, Num8,  0, "Tumble Pos Lt.",         "TMBLLT","v",        0,   0.2,   1.0 },
    {  33, 14, 6, Num8,  0, "Idle Ctrl Duty Ratio",   "IDLEDR","%",        0,   1.0,   2.0 },
    {  34, 14, 0, Num8,  0, "Fuel Pump Duty",         "FUELDC","%",        0,   1.0,   2.55 },

    {  35, 15, 7, Num8,  0, "VVT Adv. Rt.",           "VVTART","deg",     50,   1.0,   1.0 },
    {  36, 15, 6, Num8,  0, "VVT Adv. Lt.",           "VVTALT","deg",     50,   1.0,   1.0 },
    {  37, 15, 5, Num8,  0, "OCV Duty Rt.",           "OCVDRT","%",        0,   1.0,   2.55 },
    {  38, 15, 4, Num8,  0, "OCV Duty Lt.",           "OCVDLT","%",        0,   1.0,   2.55 },
    {  39, 15, 3, Num8,  0, "OCV Current Rt.",        "OCVCRT","mA",       0,  32.0,   1.0 },
    {  40, 15, 2, Num8,  0, "OCV Current Lt.",        "OCVCLT","mA",       0,  32.0,   1.0 },
    {  41, 15, 1, Num8,  0, "AFR #1 Current",         "AFR1V", "mA",     128,   0.125, 1.0 },

    {  42, 16, 7, Num8,  0, "AFR #1 Resist",          "AFR1R", "ohm",      0,   1.0,   1.0 },

    {  43, 16, 5, Num8,  0, "AFR #1",                 "AFR1",  "lambda",   0,   1.0, 128.0 },

    {  44, 16, 3, Num8,  0, "AFR Correct #3",         "AFC3",  "%",      128,   1.0,   1.28 }, // ??????????? guessed from AFC1

    {  45, 16, 1, Num8,  0, "Rear O2 Heater",         "RO2HV", "v",        0,   1.0,   1.0 },  // no idea


    {  46, 99, 1, Num8,  0, "IAM",                    "IAM",   "x",        0,   1.0,   1.0 },  

    {  47, 99, 1, Num8,  0, "LOAD",                   "LOAD",  "%",        0,   1.0,  32.0 },  

    {  48, 99, 1, Num8,  0, "Ignition Retard",        "IRTRD", "deg",      0,   1.0,   1.0 },  // not supported wrx?

    { 0 }
};


//
//  Describes the fields corresponding to memory locations in the ECU.  It is expected that
//  each field type will only be associated with one memory location, and hence a field
//  NAME will be unique within an ECU field set.
//

ECU_DESCRIPTOR ECU_GENERIC_WRX = {

    "GENERIC WRX",

    {
        //  list of ( <ecu mem addr>, <fieldid> ) pairs, where field id is first
        //  number in the ecu field entries in EcuFieldtable above.

        {   0x7, 8},
        {   0x8, 1},
        {   0x9, 2},
        {   0xa, 3},
        {   0xd, 6},
        {   0xe, 7},

        {  0x10, 9},    // speed
        {  0x11, 10},
        {  0x12, 11},
        {  0x13, 12},
        {  0x15, 13},   // throttle
        {  0x18, 15},   // 02

        {  0x1c, 17},   // battery
        {  0x1d, 18},   // maf v
        {  0x1e, 19},   // tps v

        {  0x20, 21},   // FI p/w 1
        {  0x22, 23},   // knock cr
        {  0x23, 24},   // atmos press

        {  0x24, 25},

        {  0x26, 26},

        {  0x2a, 27},   //fuel temp


        {  0x2e, 28},

        {  0x30, 29}, // pri wastegate

        {  0x32, 30}, // cpc

        {  0x33, 31}, // tumble rt


        {  0x34, 32}, // tumble lt

        {  0x35, 33}, // idle

        {  0x3b, 34}, // fuel pump


        {  0x3c, 35}, // vvt

        {  0x3d, 36}, // vvt

        {  0x3e, 37}, // ocv

        {  0x3f, 38}, // ocv

        {  0x40, 39}, // ocv

        {  0x41, 40}, // ocv

        {  0x42, 41}, // afrc


        {  0x44, 42}, // afrr

        {  0x46, 43}, // afr

        {  0x47, 44}, // ar correct 3 ???????

        {  0x48, 45}, // r02htr ??????????


        {0x20168, 46}, // IAM??  ECU revision specific.  Need to change structures.
    
    {0x200f4, 47}, // load   ECU revision specific.

        {   0, 0}
    }
};


ECU_ID_MAPPING EcuIdMappings[] = {

    { {1,2,3,4,5}, &ECU_GENERIC_WRX},

    { {0}, NULL}
};



