/*

This program and source is licensed under the

	GNU GENERAL PUBLIC LICENSE

as in the included LICENSE.TXT or located at
http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <string.h>

#include "ecucom.h"
#include "trace.h"

typedef struct _ECU_COM_CONTEXT
{
    HANDLE Handle;

    ULONG ComPort;

    BOOL Initialized;
    BOOL AllowWrites;

    //
    //  Contexts for async io.
    //

    OVERLAPPED ReadCtx;
    OVERLAPPED WriteCtx;

} ECU_COM_CONTEXT, *PECU_COM_CONTEXT;


static ECU_COM_CONTEXT State = {0};



BOOL
EcuComSend(
    PVOID Buffer,
    ULONG Bytes,
    PULONG ErrorCode
    )
{
    DWORD WaitResult;
    ULONG BytesWritten;
    ULONG LocalErr;
    BOOL Result;

    if (!State.Initialized) {

        *ErrorCode = ERROR_INVALID_PARAMETER;
        return FALSE;
    }

    //
    //  Ensure that no bugs send write requests!
    //
/*
	if (!State.AllowWrites) {

        PPACKET Packet = Buffer;

        if ((Packet->Data[0] == ECU_OP_WRITE) ||
            (Packet->Data[0] == ECU_OP_WRITE_ADDR)) {

            *ErrorCode = ERROR_INVALID_PARAMETER;
            TracePrint(TRACE_INFO,"EcuCom: Recieved write request, but writes not allowed\n");
            return FALSE;
        }
    }
*/
    TracePrint(TRACE_COM,"Sending data:\n");
    DumpHex(TRACE_COM,Buffer,Bytes);

    Result = WriteFile( State.Handle,
                        Buffer,
                        Bytes,
                        &BytesWritten,
                        &State.WriteCtx);

    if (!Result) {

        LocalErr = GetLastError();

        if (ERROR_IO_PENDING == LocalErr) {

            TracePrint(TRACE_COM, "Write pending\n");

            WaitResult = WaitForSingleObjectEx( State.WriteCtx.hEvent,
                                                -1,
                                                FALSE);


            if (WaitResult != 0) {

                TracePrint(TRACE_ERROR,"Wait returned unexpected result %d\n", WaitResult);
            }
            else {

                if (GetOverlappedResult( State.Handle,
                                         &State.WriteCtx,
                                         &BytesWritten,
                                         TRUE)) {
                    Result = TRUE;
                }
            }
        }
    }

    if (!Result) {

        *ErrorCode = GetLastError();
    }
    else if (BytesWritten != Bytes) {

        TracePrint(TRACE_ERROR,"Only wrote %d of %d bytes!\n", BytesWritten, Bytes);
        Result = FALSE;
        *ErrorCode = ERROR_INTERNAL_ERROR;
    }
    else {

        *ErrorCode = ERROR_SUCCESS;
    }

    return Result;
}


BOOL
EcuComReceive(
    PVOID InputBuffer,
    ULONG Bytes,
    PULONG ErrorCode
    )
{
    ULONG BytesRead = 0;
    LONG BytesRemaining = Bytes;
    BOOL Result;
    DWORD Events;
    DWORD Errors;
    COMSTAT ComStat;
    DWORD Trash;
    DWORD WaitResult;
    PUCHAR Buffer = InputBuffer;

    if (!State.Initialized) {

        *ErrorCode = ERROR_INVALID_PARAMETER;
        return FALSE;
    }

    do {

        //
        //  Wait for some data to arrive in the port buffer.
        //

        Result = WaitCommEvent( State.Handle, &Events, &State.ReadCtx);

        if (!Result) {

            if (GetLastError() == ERROR_IO_PENDING) {

                TracePrint(TRACE_COM, "WaitCommEvent pending\n");

                WaitResult = WaitForSingleObjectEx( State.ReadCtx.hEvent,
                                                    -1,
                                                    FALSE);
                if (WaitResult != 0) {

                    TracePrint(TRACE_ERROR,"Wait(comm) returned unexpected result %d\n", WaitResult);
                }
                else {

                    Result = GetOverlappedResult( State.Handle,
                                                  &State.ReadCtx,
                                                  &Trash,
                                                  TRUE);
                }
            }
        }

        if (!Result) {

            *ErrorCode = GetLastError();
            TracePrint(TRACE_ERROR,"Failed WaitCommEvent, error %d\n", *ErrorCode );
            break;
        }

        TracePrint( TRACE_COM, "Woke on event mask 0x%x\n", Events);

        if (Events & EV_ERR) {

            TracePrint(TRACE_ERROR,"Receive error (0x%x)\n", Events);
            Result = FALSE;
            *ErrorCode = ERROR_BUS_RESET;
            break;
        }

        //
        //  Find out how many bytes are in the buffer to be read.
        //

        if (!ClearCommError( State.Handle, &Errors, &ComStat)) {

            TracePrint(TRACE_ERROR,"Failed to clear error, error %d\n", GetLastError());
            break;
        }

        TracePrint( TRACE_COM, "Inb Outb %d %d\n", ComStat.cbInQue, ComStat.cbOutQue);

        if (0 != ComStat.cbInQue) {

            Result = ReadFile( State.Handle,
                               Buffer,
                               ComStat.cbInQue,
                               &BytesRead,
                               &State.ReadCtx);

            if (!Result && (ERROR_IO_PENDING == GetLastError())) {

                TracePrint(TRACE_COM, "Read pending\n");

                WaitResult = WaitForSingleObjectEx( State.ReadCtx.hEvent,
                                                    -1,
                                                    FALSE);
                if (WaitResult != 0) {

                    TracePrint(TRACE_ERROR,"Wait(read) returned unexpected result %d\n", WaitResult);
                }
                else {

                    Result = GetOverlappedResult( State.Handle,
                                                  &State.ReadCtx,
                                                  &BytesRead,
                                                  TRUE);
                }
            }

            if (!Result) {

                *ErrorCode = GetLastError();
                TracePrint(TRACE_ERROR,"Readfile failed, error %d\n", *ErrorCode);
                break;
            }
            else if (BytesRead < ComStat.cbInQue) {

                TracePrint(TRACE_ERROR,"Readfile only returned %d, expecting %d, probably timeout\n", BytesRead, ComStat.cbInQue);
                Result = FALSE;
                *ErrorCode = ERROR_NO_DATA;
                break;
            }

            BytesRemaining -= BytesRead;
            Buffer += BytesRead;

            TracePrint( TRACE_COM, "Remaining %d\n", BytesRemaining);
        }
        else {

            TracePrint( TRACE_COM, "No bytes in input queue, rewaiting\n");
        }
    }
    while (BytesRemaining > 0);

    return Result;
}


BOOL
EcuComInitialize(
    ULONG ComPort
    )
{
    UCHAR Path[32];
    DWORD EventMask;
    DCB Dcb;
    BOOL Result = FALSE;
    COMMTIMEOUTS TimeOuts;

    if (State.Initialized) {

        return TRUE;
    }

    State.ComPort = ComPort;

    sprintf( Path, "\\\\.\\com%d", ComPort);
    TracePrint(TRACE_INFO,"Opening %s\n", Path);

    State.Handle = CreateFile( Path,
                               GENERIC_READ | GENERIC_WRITE,
                               0,
                               NULL,
                               OPEN_EXISTING,
                               FILE_FLAG_OVERLAPPED, //0,//FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH,
                               NULL);

    if (INVALID_HANDLE_VALUE == State.Handle) {

        TracePrint(TRACE_ERROR,"Failed to open com port, error %d\n", GetLastError());
        return FALSE;
    }

    EcuComPurge();

    //
    //  Ignore all events.
    //

    EventMask = EV_RXCHAR;

    if (!SetCommMask( State.Handle, EventMask)) {

        TracePrint(TRACE_ERROR,"Failed to set event mask, error %d\n", GetLastError());
        goto Exit;
    }

    //
    //  Set communication parameters.
    //

    if (!GetCommState( State.Handle, &Dcb)) {

        TracePrint(TRACE_ERROR,"Failed to get comm state, error %d\n", GetLastError());
        goto Exit;
    }

    Dcb.DCBlength = sizeof( Dcb);
    Dcb.BaudRate = 4800;
    Dcb.fBinary = 1;
    Dcb.fParity = 0;
    Dcb.Parity = NOPARITY;
    Dcb.StopBits = ONESTOPBIT;
    Dcb.ByteSize = 8;

    Dcb.fRtsControl = RTS_CONTROL_DISABLE;
    Dcb.fDtrControl = DTR_CONTROL_ENABLE;

    Dcb.fInX = 0;
    Dcb.fOutX = 0;

    if (!SetCommState( State.Handle, &Dcb)) {

        TracePrint(TRACE_ERROR,"Failed to set comm state, error %d\n", GetLastError());
        goto Exit;
    }

    //
    //  Set timeouts.
    //

    memset( &TimeOuts, 0, sizeof( TimeOuts));

    TimeOuts.ReadIntervalTimeout = 1000;            // Max 1000ms between bytes (1sec)
    TimeOuts.ReadTotalTimeoutMultiplier = 0;
    TimeOuts.ReadTotalTimeoutConstant = 5*1000;     // Total timeout 5 secs per read

    TimeOuts.WriteTotalTimeoutMultiplier = 0;
    TimeOuts.WriteTotalTimeoutConstant = 5*1000;    // 5 seconds max wait.

    if (!SetCommTimeouts( State.Handle, &TimeOuts)) {

        TracePrint(TRACE_ERROR,"Failed to set comm timeouts, error %d\n", GetLastError());
        goto Exit;
    }

    Result = TRUE;

    TracePrint(TRACE_INFO,"Successfully configured com port\n");

    //
    //  Create events for the async IO operations
    //

    memset( &State.WriteCtx, 0, sizeof( State.WriteCtx));
    memset( &State.ReadCtx, 0, sizeof( State.ReadCtx));

    State.ReadCtx.hEvent = CreateEvent( NULL, FALSE, FALSE, NULL);
    State.WriteCtx.hEvent = CreateEvent( NULL, FALSE, FALSE, NULL);

    EcuComPurge();

    State.Initialized = TRUE;

    State.AllowWrites = FALSE;
Exit:

    if (!Result) {

        CloseHandle( State.Handle);
        State.Handle = INVALID_HANDLE_VALUE;
    }

    return Result;
}


VOID
EcuComUnInitialize(
    )
{
    if (!State.Initialized) {

        return;
    }

    CloseHandle( State.Handle);
    CloseHandle( State.WriteCtx.hEvent);
    CloseHandle( State.ReadCtx.hEvent);

    State.Handle = INVALID_HANDLE_VALUE;
    State.Initialized = FALSE;
}


VOID
EcuComPurge(
    )
{
    if (!State.Initialized) {

        return;
    }

    PurgeComm( State.Handle, PURGE_TXCLEAR | PURGE_RXCLEAR | PURGE_TXABORT | PURGE_RXABORT);
}



