/*

This program and source is licensed under the

    GNU GENERAL PUBLIC LICENSE

as in the included LICENSE.TXT or located at
http://www.gnu.org/licenses/gpl.txt

*/

#ifndef __ECU_CORE_H
#define __ECU_CORE_H

#include "ecu.h"

BOOL
EcuCoreReceivePacket(
    IN PVOID Buffer,
    IN ULONG ReceiveDataLen,
    OUT PPACKET *ResponsePacket
    );

ULONG
EcuCorePreparePacket(
    IN OUT PPACKET_WBUF PacketBuffer
    );

BOOL
EcuCoreConnectEcu(
    OUT PECU_DESCRIPTOR *Ecu,
    OUT PUCHAR *InitResponseData
    );

BOOL
EcuCoreInit(
    IN ULONG ComPort
    );


VOID
EcuCoreUnInit(
    );


ULONG
EcuCoreFieldIdFromFieldShortName(,
    IN PECU_DESCRIPTOR Ecu,
    IN UCHAR ShortName[]
    );

PUCHAR
EcuCoreUnitsFromFieldId(
    IN ULONG FieldId
    );

//
//  Structures and functions for data collection
//

#define ECU_CORE_INVALID_FIELD_ID           0

//
//  Max fields in read address = (max packet data length - READ operation code - padding) / address size
//
//  Some field types are 16bits = 2 addresses in read.
//

#define ECU_MAX_SAMPLE_FIELDS               ((255 - 2) / ECU_ADDR_LEN)

typedef struct _ECU_CORE_SAMPLE_DESCRIPTOR {

    //
    //  Input fields
    //

    ULONG FieldId;
    ULONG Frequency;  // 1 = every sample, 2 = every other sample, ....

    //
    //  Output fields
    //

    double Value;
    BOOL UpdatedThisSample;

    //
    //  ECUCORE internal fields (used during collection).
    //

    ULONG EcuAddress;
    PECU_FIELD_DESCRIPTOR Field;
}
ECU_CORE_SAMPLE_DESCRIPTOR, *PECU_CORE_SAMPLE_DESCRIPTOR;

//
//  This structure is used to define fields to be sampled by data collector, and also to receive
//  the collected data.
//

typedef struct _ECU_CORE_SAMPLE_SET {

    //  input

    ULONG NumFieldsToSample;

    //  input and output

    ECU_CORE_SAMPLE_DESCRIPTOR SampleList[ ECU_MAX_SAMPLE_FIELDS];

    //  output - updated for each sample returned

    ULONG TimeStamp;    // in ms, since data collection start
    ULONG SampleNumber;
}
ECU_CORE_SAMPLE_SET, *PECU_CORE_SAMPLE_SET;


typedef
BOOL
(*PECU_CORE_SAMPLE_NOTIFY_CALLBACK) (
    VOID
    );


BOOL
EcuCoreCollectData(
    IN     PECU_CORE_SAMPLE_SET SampleSet,
    IN     PECU_CORE_SAMPLE_NOTIFY_CALLBACK SampleNotifyCallBack
    );
//
//  Samplenotifycallback is called when data has been placed in SampleData, callback
//  in same caller thread whch called EcuCoreCollectData (i.e. this function will
//  not return until the sample callback routine returns FALSE to end sampling).
//
//  To end sampling, this routine should return FALSE, TRUE to continue sampling.
//


BOOL
EcuCoreReadMemory(
    IN ULONG ECUAddress,
    IN UCHAR Length,
    IN OUT PVOID Buffer
    );

#endif

