/*

This program and source is licensed under the

    GNU GENERAL PUBLIC LICENSE

as in the included LICENSE.TXT or located at
http://www.gnu.org/licenses/gpl.txt

*/


#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#include "ecucore.h"


//
//  State - ECU context.
//

PECU_DESCRIPTOR Ecu;
PUCHAR InitResponseData;

//
//  Our data logging state / parameter list.
//

ECU_CORE_SAMPLE_SET SampleSet;
PECU_CORE_SAMPLE_DESCRIPTOR Samples = SampleSet.SampleList;
FILE *LogFile;
ULONG LogCount;



VOID
ReadMemory(
    PUCHAR Param[]
    )
{
    UCHAR Buffer[256];
    ULONG I;
	ULONG Length = atoi( Param[1]);

    if (EcuCoreReadMemory( atoi( Param[0]), (UCHAR)Length, Buffer)) {

        for (I = 0; I < Length; ++I) {

            if (0 == (I % 16)) {

                printf("\n");
            }

            printf("%0.2x ", Buffer[I]);
        }

        printf("\n");
    }
}


BOOL
LogCallBack(
    VOID
    )
{
    ULONG I;

    fprintf(LogFile,"%.2f\t", ((float)(SampleSet.TimeStamp)) / 1000);

    for (I = 0; I < SampleSet.NumFieldsToSample; ++I) {

        if (Samples[I].UpdatedThisSample) {

            fprintf(LogFile, "%.2f\t", Samples[I].Value);
        }
        else {

            fprintf( LogFile, " \t");
        }
    }

    fprintf(LogFile,"\n");

    LogCount++;

    //
    //  Flush the logfile every 20 samples or so.
    //

    if (0 == LogCount % 20){

        fflush(LogFile);
    }

	//
	//	Life indicator every 5 seconds
	//

	if (0 == ((SampleSet.TimeStamp / 1000) % 5)) {

		printf("LOGGING: Collected %d samples so far\n", SampleSet.SampleNumber);
	}

    return TRUE;
}



VOID
Log(
    PUCHAR OutputFileName,
    PUCHAR ParamsToLog[],
    ULONG ParamCount
    )
{
    ULONG I;
    PUCHAR Name;
    ULONG Frequency;
    ULONG FieldId;
    ULONG NumSamples;

    memset( Samples, 0, sizeof(Samples));
    NumSamples = 0;

    //
    //  Open logfile and print header
    //

    LogFile = fopen( OutputFileName, "w+");

    if (NULL == LogFile) {

        printf("Failed to open logfile\n");
        return;
    }

    fprintf(LogFile,"TIME/Sec\t");

    //
    //  Walk the user supplied log parameters and map them to IDs present in the current ECU
    //

    for (I = 0; I < ParamCount; ++I) {

        Name = ParamsToLog[I];

        //
        //  Check for a :X: sample frequency prefix
        //

        if (Name[0] == ':') {

            Frequency = atoi( ParamsToLog[I] + 1);

            Name += 1;

            while (':' != Name[0]) {

                Name += 1;
            }

            Name += 1;
        }
        else {

            Frequency = 1;
        }

        printf("Every %d samples, field \'%s\' ", Frequency, Name);

        //
        //  Lookup the name
        //

        FieldId = EcuCoreFieldIdFromFieldShortName( Ecu, Name);

        if (FieldId == ECU_CORE_INVALID_FIELD_ID) {

            printf("Field name invalid / not mapped for this ECU\n");
            fclose(LogFile);
            LogFile = NULL;
            return;
        }

        printf("- Ok (Id %d)\n", FieldId);

        Samples[ NumSamples].FieldId = FieldId;
        Samples[ NumSamples].Frequency = Frequency;

        NumSamples += 1;

        //
        //  Print logfile header entry
        //

        fprintf(LogFile,"%s/%s\t", Name, EcuCoreUnitsFromFieldId( FieldId));
    }

    fprintf(LogFile,"\n");

    LogCount = 0;

    SampleSet.NumFieldsToSample = NumSamples;

    //
    //  Now go sample the data!
    //

	printf("Starting logging... CTRL+C to stop\n");

    if (!EcuCoreCollectData( &SampleSet, LogCallBack)) {

        printf("Data collection failed\n");
    }

    fclose(LogFile);
    LogFile = NULL;
}


VOID
DumpRom(
    PUCHAR FileName
    )
{
}


VOID
GetFieldName(
    IN ULONG Byte,
    IN ULONG Bit,
    OUT PUCHAR *ShortName,
    OUT PUCHAR *Name
    )
{
    ULONG I = 0;

    *ShortName = "?????";
    *Name = "Unknown";

    do {

        if ((EcuFieldTable[ I].InitResponseSupportedByte == Byte) &&
            (EcuFieldTable[ I].InitResponseSupportedBit == Bit)) {

            *Name = EcuFieldTable[ I].FieldLongName;
            *ShortName = EcuFieldTable[ I].FieldShortName;
            break;
        }

        ++I;

    } while (0 != EcuFieldTable[ I].FieldID);
}


VOID
DumpEcuInfo()
{
    PECU_INIT_RESPONSE InitData = (PVOID)InitResponseData;
    ULONG Byte, I;
    LONG Bit;
    PUCHAR Name, ShortName;

    printf("\nECUID: ");

    for (I = 0; I < 5; ++I) {

        printf("%0.2x ", InitData->EcuID[I]);
    }

    printf("\nMapped to descriptor \'%s\'\n", Ecu->Name);

    printf("\n\nSupported parameters\n\nByte  Bit\n\n");

    for (Byte = 0; Byte < ECU_INIT_RESPONSE_INIT_DATA_LEN; Byte++) {

        for (Bit = 7; Bit >= 0; Bit -= 1) {

            if (InitData->Data[ Byte] & (1 << Bit)) {

                GetFieldName( Byte+9, Bit, &ShortName, &Name);

                printf("0x%0.3x %d   %5s %s\n", Byte, Bit, ShortName, Name);
            }
        }
    }
}


VOID
PrintArgs( VOID)
{
    printf("\nWRXUTIL <comport#> <...parameters...>\n"
           " \n"
           "where valid parameters are:\n"
           " \n"
           "  INFO\n"
           "  DUMPROM <filename>\n"
           "  READMEM <address> <length>\n"
           "  LOG <outputfilename> [:freq:]<param> [[:freq:]<param>, ...]]\n\n");
}


INT
_cdecl
main(
    INT argc,
    UCHAR *argv[]
    )
{
    ULONG I = 0;

    printf("\nWRX ECU Util v1.03 (6/7/05)\n\nby realwomble based on info @ www.openecu.org\n");
    printf("Check there for sources and updates. \n\nThis program and sources are licensed based on the\n\n");
    printf("    GNU GENERAL PUBLIC LICENSE (see LICENSE.TXT)\n    (http://www.gnu.org/licenses/gpl.txt)\n\n");

    if (argc < 3) {

        PrintArgs();
        exit(1);
    }

    if (!EcuCoreInit( atoi( argv[1]))) {

        printf("ECU init failed\n");
        exit(1);
    }

    if (!EcuCoreConnectEcu( &Ecu, &InitResponseData)) {

        printf("ECU connect failed\n");
        exit(1);
    }

    if (!_stricmp( "DUMPROM", argv[2])) {

        DumpRom( argv[3]);
    }
    else if (!_stricmp("READMEM", argv[2])) {

        ReadMemory( &argv[3]);
    }
    else if (!_stricmp("LOG", argv[2])) {

        //
        //  Datalogging.
        //

        if (argc < 5) {

            PrintArgs();
            exit(1);
        }

        Log( argv[3], &argv[4], argc - 4);
    }
    else if (!_stricmp("INFO", argv[2])) {

        DumpEcuInfo();
    }
    else {

        printf("Unknown parameter \'%s\'\n", argv[2]);
        exit(1);
    }

    EcuCoreUnInit();

    return 0;
}


